{*******************************************************}
{                                                       }
{  VG Library II                                        }
{                                                       }
{  VCL Design Editors                                   }
{                                                       }
{  (c) 1997, 2001 by Vladimir Gaitanoff                 }
{                                                       }
{*******************************************************}

{$I vg2.inc}
{$D-,L-}

unit vg2VclDsgn;

interface
uses DsgnIntf, vg2PropStore;

type
{ TCustomEditEditor }

  TCustomEditEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TCustomDialogEditor }

  TCustomDialogEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TLauncherEditor }

  TLauncherEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TMultiLanguageEditor }

  TMultiLanguageEditor = class (TComponentEditor)
  private
    procedure CreateLanguageFile;
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TPictureMaskProperty }

  TPictureMaskProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

{ TLaunchParamsProperty }

  TLaunchParamsProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

{ TListNamesProperty }

  TListNamesProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
  end;

{ TFileNameProperty }

  TFileNameProperty = class(TStringProperty)
  protected
    function GetFilter: string; dynamic;
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

{ TDirectoryProperty }

  TDirectoryProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

{ THintProperty }

  THintProperty = class(TCaptionProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetEditLimit: Integer; override;
    procedure Edit; override;
  end;

var
  AppIniFile: Tvg2AppIniFile;

implementation
uses TypInfo, SysUtils, Classes, Controls, Dialogs,
  vg2Consts, vg2Tools, vg2VclUtils, vg2StdCtrls, vg2FileCtrls, vg2MultiLang,
  vg2MaskDs, vg2LaunchDs, vg2StrEdit;

{ TCustomEditEditor }

type
  Tvg2CustomEditHack = class(Tvg2CustomEdit);
  
procedure TCustomEditEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0:
      begin
        AppIniFile.BeginWrite;
        try
          if EditPicture(Tvg2CustomEditHack(Component).Picture, AppIniFile.IniFile) then
            Designer.Modified;
        finally
          AppIniFile.EndWrite;
        end;
      end;
  else
      inherited ExecuteVerb(Index - 1);
  end;
end;

function TCustomEditEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0:
      Result := SEditPicture;
  else
      Result := inherited GetVerb(Index - 1);
  end;
end;

function TCustomEditEditor.GetVerbCount: Integer;
begin
  Result := inherited GetVerbCount + 1;
end;

{ TCustomDialogEditor }

procedure TCustomDialogEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0:
      begin
        Tvg2CustomDialog(Component).Execute;
        Designer.Modified;
      end;
  else
      inherited ExecuteVerb(Index - 1);
  end;
end;

function TCustomDialogEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0:
      Result := STestDialog;
  else
      Result := inherited GetVerb(Index - 1);
  end;
end;

function TCustomDialogEditor.GetVerbCount: Integer;
begin
  Result := inherited GetVerbCount + 1;
end;

{ TLauncherEditor }

procedure TLauncherEditor.ExecuteVerb(Index: Integer);
var
  Obj: TObject;
begin
  case Index of
    0:
      begin
        Obj := GetObjectProp(Component, 'LaunchParams', Tvg2LaunchParams);
        if Assigned(Obj) and EditLaunchParams(Tvg2LaunchParams(Obj)) then
          Designer.Modified;
      end;
    1:
      begin
        Obj := GetObjectProp(Component, 'LaunchParams', Tvg2LaunchParams);
        if Assigned(Obj) then Tvg2LaunchParams(Obj).Execute;
      end;
  else
      inherited ExecuteVerb(Index - 1);
  end;
end;

function TLauncherEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0:
      Result := SEdit;
    1:
      Result := SExecute;
  else
      Result := inherited GetVerb(Index - 1);
  end;
end;

function TLauncherEditor.GetVerbCount: Integer;
begin
  Result := inherited GetVerbCount + 2;
end;

{ TMultiLanguageEditor }

procedure TMultiLanguageEditor.CreateLanguageFile;
var
  FileOpen: TOpenDialog;
begin
  FileOpen := TSaveDialog.Create(nil);
  try
    FileOpen.FileName := Tvg2MultiLanguage(Component).FileName;
    FileOpen.InitialDir := ExtractFilePath(FileOpen.FileName);
    if (ExtractFileName(FileOpen.FileName) = '') then
      FileOpen.FileName := '';
    FileOpen.Filter := 'Ini files (*.ini)|*.ini|All files (*.*)|*.*';
    FileOpen.DefaultExt := 'ini';
    FileOpen.Options := FileOpen.Options + [ofHideReadOnly];
    if FileOpen.Execute then
    begin
      AppSetCursor(crHourglass);
      try
        Tvg2MultiLanguage(Component).CreateLanguageFile(FileOpen.FileName, True);
      finally
        AppRestoreCursor;
      end;
    end;
  finally
    FileOpen.Free;
  end;
end;

procedure TMultiLanguageEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: CreateLanguageFile;
  end;
end;

function TMultiLanguageEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := SCreateLanguageFile
  end;
end;

function TMultiLanguageEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

{ TPictureMaskProperty }

procedure TPictureMaskProperty.Edit;
var
  I: Integer;
  Obj, Obj2: TObject;
begin
  Obj := GetObjectProp(GetComponent(0), GetName, Tvg2PictureMask);
  AppIniFile.BeginWrite;
  try
    if Assigned(Obj) and EditPicture(Tvg2PictureMask(Obj), AppIniFile.IniFile) then
    begin
      for I := 1 to PropCount - 1 do
      begin
        Obj2 := GetObjectProp(GetComponent(I), GetName, Tvg2PictureMask);
        if Assigned(Obj2) then
          Tvg2PictureMask(Obj2).Assign(TPersistent(Obj2));
      end;
      Designer.Modified;
    end;
  finally
    AppIniFile.EndWrite;
  end;
end;

function TPictureMaskProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog];
end;

{ TLaunchParamsProperty }

procedure TLaunchParamsProperty.Edit;
var
  I: Integer;
  Obj, Obj2: TObject;
begin
  Obj := GetObjectProp(GetComponent(0), GetName, Tvg2LaunchParams);
  if Assigned(Obj) and EditLaunchParams(Tvg2LaunchParams(Obj)) then
  begin
    for I := 1 to PropCount - 1 do
    begin
      Obj2 := GetObjectProp(GetComponent(I), GetName, Tvg2LaunchParams);
      if Assigned(Obj2) then
        Tvg2LaunchParams(Obj2).Assign(TPersistent(Obj2));
    end;
    Designer.Modified;
  end;
end;

function TLaunchParamsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog];
end;

{ TListNamesProperty }

function TListNamesProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect, paRevertable];
end;

{ TFileNameProperty }

function TFileNameProperty.GetFilter: string;
begin
  Result := '*.*';
end;

procedure TFileNameProperty.Edit;
var
  FileOpen: TOpenDialog;
begin
  FileOpen := TOpenDialog.Create(nil);
  try
    FileOpen.FileName := GetValue;
    FileOpen.InitialDir := ExtractFilePath(FileOpen.FileName);
    FileOpen.FileName := ExtractFileName(FileOpen.FileName);
    FileOpen.Filter := GetFilter;
    FileOpen.Options := FileOpen.Options + [ofHideReadOnly];
    if FileOpen.Execute then SetValue(FileOpen.FileName);
  finally
    FileOpen.Free;
  end;
end;

function TFileNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paRevertable];
end;

{ TDirectoryProperty }

procedure TDirectoryProperty.Edit;
var
  I: Integer;
  Dir: TFileName;
begin
  Dir := GetStrProp(GetComponent(0), GetName);
  if BrowseForFolder(Dir, bfDirectory, '', 0) then
  begin
    for I := 0 to PropCount - 1 do
      SetStrProp(GetComponent(I), GetName, Dir);
    Designer.Modified;
  end;
end;

function TDirectoryProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog];
end;

function THintProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog];
end;

function THintProperty.GetEditLimit: Integer;
begin
  if GetPropType^.Kind = tkString then
    Result := GetTypeData(GetPropType)^.MaxLength else
    Result := 1024;
end;

procedure THintProperty.Edit;
var
  Temp: string;
begin
  Temp := GetStrValue;
  if EditText(GetComponent(0), GetName, Temp) then
    SetStrValue(Temp);
end;

initialization
  AppIniFile := Tvg2AppIniFile.Create(nil);
  AppIniFile.RegistryRoot := rrLocalMachine;
  AppIniFile.IniFileName := SVGLibKey;

finalization
  FreeAndNil(AppIniFile);


end.
